/*******************************************************************************
* File Name: UARTOUT.h
* Version 2.50
*
* Description:
*  Contains the function prototypes and constants available to the UART
*  user module.
*
* Note:
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/


#if !defined(CY_UART_UARTOUT_H)
#define CY_UART_UARTOUT_H

#include "cytypes.h"
#include "cyfitter.h"
#include "CyLib.h"


/***************************************
* Conditional Compilation Parameters
***************************************/

#define UARTOUT_RX_ENABLED                     (0u)
#define UARTOUT_TX_ENABLED                     (1u)
#define UARTOUT_HD_ENABLED                     (0u)
#define UARTOUT_RX_INTERRUPT_ENABLED           (0u)
#define UARTOUT_TX_INTERRUPT_ENABLED           (0u)
#define UARTOUT_INTERNAL_CLOCK_USED            (0u)
#define UARTOUT_RXHW_ADDRESS_ENABLED           (0u)
#define UARTOUT_OVER_SAMPLE_COUNT              (8u)
#define UARTOUT_PARITY_TYPE                    (0u)
#define UARTOUT_PARITY_TYPE_SW                 (0u)
#define UARTOUT_BREAK_DETECT                   (0u)
#define UARTOUT_BREAK_BITS_TX                  (13u)
#define UARTOUT_BREAK_BITS_RX                  (13u)
#define UARTOUT_TXCLKGEN_DP                    (1u)
#define UARTOUT_USE23POLLING                   (1u)
#define UARTOUT_FLOW_CONTROL                   (0u)
#define UARTOUT_CLK_FREQ                       (0u)
#define UARTOUT_TX_BUFFER_SIZE                 (4u)
#define UARTOUT_RX_BUFFER_SIZE                 (4u)

/* Check to see if required defines such as CY_PSOC5LP are available */
/* They are defined starting with cy_boot v3.0 */
#if !defined (CY_PSOC5LP)
    #error Component UART_v2_50 requires cy_boot v3.0 or later
#endif /* (CY_PSOC5LP) */

#if defined(UARTOUT_BUART_sCR_SyncCtl_CtrlReg__CONTROL_REG)
    #define UARTOUT_CONTROL_REG_REMOVED            (0u)
#else
    #define UARTOUT_CONTROL_REG_REMOVED            (1u)
#endif /* End UARTOUT_BUART_sCR_SyncCtl_CtrlReg__CONTROL_REG */


/***************************************
*      Data Structure Definition
***************************************/

/* Sleep Mode API Support */
typedef struct UARTOUT_backupStruct_
{
    uint8 enableState;

    #if(UARTOUT_CONTROL_REG_REMOVED == 0u)
        uint8 cr;
    #endif /* End UARTOUT_CONTROL_REG_REMOVED */

} UARTOUT_BACKUP_STRUCT;


/***************************************
*       Function Prototypes
***************************************/

void UARTOUT_Start(void) ;
void UARTOUT_Stop(void) ;
uint8 UARTOUT_ReadControlRegister(void) ;
void UARTOUT_WriteControlRegister(uint8 control) ;

void UARTOUT_Init(void) ;
void UARTOUT_Enable(void) ;
void UARTOUT_SaveConfig(void) ;
void UARTOUT_RestoreConfig(void) ;
void UARTOUT_Sleep(void) ;
void UARTOUT_Wakeup(void) ;

/* Only if RX is enabled */
#if( (UARTOUT_RX_ENABLED) || (UARTOUT_HD_ENABLED) )

    #if (UARTOUT_RX_INTERRUPT_ENABLED)
        #define UARTOUT_EnableRxInt()  CyIntEnable (UARTOUT_RX_VECT_NUM)
        #define UARTOUT_DisableRxInt() CyIntDisable(UARTOUT_RX_VECT_NUM)
        CY_ISR_PROTO(UARTOUT_RXISR);
    #endif /* UARTOUT_RX_INTERRUPT_ENABLED */

    void UARTOUT_SetRxAddressMode(uint8 addressMode)
                                                           ;
    void UARTOUT_SetRxAddress1(uint8 address) ;
    void UARTOUT_SetRxAddress2(uint8 address) ;

    void  UARTOUT_SetRxInterruptMode(uint8 intSrc) ;
    uint8 UARTOUT_ReadRxData(void) ;
    uint8 UARTOUT_ReadRxStatus(void) ;
    uint8 UARTOUT_GetChar(void) ;
    uint16 UARTOUT_GetByte(void) ;
    uint8 UARTOUT_GetRxBufferSize(void)
                                                            ;
    void UARTOUT_ClearRxBuffer(void) ;

    /* Obsolete functions, defines for backward compatible */
    #define UARTOUT_GetRxInterruptSource   UARTOUT_ReadRxStatus

#endif /* End (UARTOUT_RX_ENABLED) || (UARTOUT_HD_ENABLED) */

/* Only if TX is enabled */
#if(UARTOUT_TX_ENABLED || UARTOUT_HD_ENABLED)

    #if(UARTOUT_TX_INTERRUPT_ENABLED)
        #define UARTOUT_EnableTxInt()  CyIntEnable (UARTOUT_TX_VECT_NUM)
        #define UARTOUT_DisableTxInt() CyIntDisable(UARTOUT_TX_VECT_NUM)
        #define UARTOUT_SetPendingTxInt() CyIntSetPending(UARTOUT_TX_VECT_NUM)
        #define UARTOUT_ClearPendingTxInt() CyIntClearPending(UARTOUT_TX_VECT_NUM)
        CY_ISR_PROTO(UARTOUT_TXISR);
    #endif /* UARTOUT_TX_INTERRUPT_ENABLED */

    void UARTOUT_SetTxInterruptMode(uint8 intSrc) ;
    void UARTOUT_WriteTxData(uint8 txDataByte) ;
    uint8 UARTOUT_ReadTxStatus(void) ;
    void UARTOUT_PutChar(uint8 txDataByte) ;
    void UARTOUT_PutString(const char8 string[]) ;
    void UARTOUT_PutArray(const uint8 string[], uint8 byteCount)
                                                            ;
    void UARTOUT_PutCRLF(uint8 txDataByte) ;
    void UARTOUT_ClearTxBuffer(void) ;
    void UARTOUT_SetTxAddressMode(uint8 addressMode) ;
    void UARTOUT_SendBreak(uint8 retMode) ;
    uint8 UARTOUT_GetTxBufferSize(void)
                                                            ;
    /* Obsolete functions, defines for backward compatible */
    #define UARTOUT_PutStringConst         UARTOUT_PutString
    #define UARTOUT_PutArrayConst          UARTOUT_PutArray
    #define UARTOUT_GetTxInterruptSource   UARTOUT_ReadTxStatus

#endif /* End UARTOUT_TX_ENABLED || UARTOUT_HD_ENABLED */

#if(UARTOUT_HD_ENABLED)
    void UARTOUT_LoadRxConfig(void) ;
    void UARTOUT_LoadTxConfig(void) ;
#endif /* End UARTOUT_HD_ENABLED */


/* Communication bootloader APIs */
#if defined(CYDEV_BOOTLOADER_IO_COMP) && ((CYDEV_BOOTLOADER_IO_COMP == CyBtldr_UARTOUT) || \
                                          (CYDEV_BOOTLOADER_IO_COMP == CyBtldr_Custom_Interface))
    /* Physical layer functions */
    void    UARTOUT_CyBtldrCommStart(void) CYSMALL ;
    void    UARTOUT_CyBtldrCommStop(void) CYSMALL ;
    void    UARTOUT_CyBtldrCommReset(void) CYSMALL ;
    cystatus UARTOUT_CyBtldrCommWrite(const uint8 pData[], uint16 size, uint16 * count, uint8 timeOut) CYSMALL
             ;
    cystatus UARTOUT_CyBtldrCommRead(uint8 pData[], uint16 size, uint16 * count, uint8 timeOut) CYSMALL
             ;

    #if (CYDEV_BOOTLOADER_IO_COMP == CyBtldr_UARTOUT)
        #define CyBtldrCommStart    UARTOUT_CyBtldrCommStart
        #define CyBtldrCommStop     UARTOUT_CyBtldrCommStop
        #define CyBtldrCommReset    UARTOUT_CyBtldrCommReset
        #define CyBtldrCommWrite    UARTOUT_CyBtldrCommWrite
        #define CyBtldrCommRead     UARTOUT_CyBtldrCommRead
    #endif  /* (CYDEV_BOOTLOADER_IO_COMP == CyBtldr_UARTOUT) */

    /* Byte to Byte time out for detecting end of block data from host */
    #define UARTOUT_BYTE2BYTE_TIME_OUT (25u)
    #define UARTOUT_PACKET_EOP         (0x17u) /* End of packet defined by bootloader */
    #define UARTOUT_WAIT_EOP_DELAY     (5u)    /* Additional 5ms to wait for End of packet */
    #define UARTOUT_BL_CHK_DELAY_MS    (1u)    /* Time Out quantity equal 1mS */

#endif /* CYDEV_BOOTLOADER_IO_COMP */


/***************************************
*          API Constants
***************************************/
/* Parameters for SetTxAddressMode API*/
#define UARTOUT_SET_SPACE      (0x00u)
#define UARTOUT_SET_MARK       (0x01u)

/* Status Register definitions */
#if( (UARTOUT_TX_ENABLED) || (UARTOUT_HD_ENABLED) )
    #if(UARTOUT_TX_INTERRUPT_ENABLED)
        #define UARTOUT_TX_VECT_NUM            (uint8)UARTOUT_TXInternalInterrupt__INTC_NUMBER
        #define UARTOUT_TX_PRIOR_NUM           (uint8)UARTOUT_TXInternalInterrupt__INTC_PRIOR_NUM
    #endif /* UARTOUT_TX_INTERRUPT_ENABLED */

    #define UARTOUT_TX_STS_COMPLETE_SHIFT          (0x00u)
    #define UARTOUT_TX_STS_FIFO_EMPTY_SHIFT        (0x01u)
    #define UARTOUT_TX_STS_FIFO_NOT_FULL_SHIFT     (0x03u)
    #if(UARTOUT_TX_ENABLED)
        #define UARTOUT_TX_STS_FIFO_FULL_SHIFT     (0x02u)
    #else /* (UARTOUT_HD_ENABLED) */
        #define UARTOUT_TX_STS_FIFO_FULL_SHIFT     (0x05u)  /* Needs MD=0 */
    #endif /* (UARTOUT_TX_ENABLED) */

    #define UARTOUT_TX_STS_COMPLETE            (uint8)(0x01u << UARTOUT_TX_STS_COMPLETE_SHIFT)
    #define UARTOUT_TX_STS_FIFO_EMPTY          (uint8)(0x01u << UARTOUT_TX_STS_FIFO_EMPTY_SHIFT)
    #define UARTOUT_TX_STS_FIFO_FULL           (uint8)(0x01u << UARTOUT_TX_STS_FIFO_FULL_SHIFT)
    #define UARTOUT_TX_STS_FIFO_NOT_FULL       (uint8)(0x01u << UARTOUT_TX_STS_FIFO_NOT_FULL_SHIFT)
#endif /* End (UARTOUT_TX_ENABLED) || (UARTOUT_HD_ENABLED)*/

#if( (UARTOUT_RX_ENABLED) || (UARTOUT_HD_ENABLED) )
    #if(UARTOUT_RX_INTERRUPT_ENABLED)
        #define UARTOUT_RX_VECT_NUM            (uint8)UARTOUT_RXInternalInterrupt__INTC_NUMBER
        #define UARTOUT_RX_PRIOR_NUM           (uint8)UARTOUT_RXInternalInterrupt__INTC_PRIOR_NUM
    #endif /* UARTOUT_RX_INTERRUPT_ENABLED */
    #define UARTOUT_RX_STS_MRKSPC_SHIFT            (0x00u)
    #define UARTOUT_RX_STS_BREAK_SHIFT             (0x01u)
    #define UARTOUT_RX_STS_PAR_ERROR_SHIFT         (0x02u)
    #define UARTOUT_RX_STS_STOP_ERROR_SHIFT        (0x03u)
    #define UARTOUT_RX_STS_OVERRUN_SHIFT           (0x04u)
    #define UARTOUT_RX_STS_FIFO_NOTEMPTY_SHIFT     (0x05u)
    #define UARTOUT_RX_STS_ADDR_MATCH_SHIFT        (0x06u)
    #define UARTOUT_RX_STS_SOFT_BUFF_OVER_SHIFT    (0x07u)

    #define UARTOUT_RX_STS_MRKSPC           (uint8)(0x01u << UARTOUT_RX_STS_MRKSPC_SHIFT)
    #define UARTOUT_RX_STS_BREAK            (uint8)(0x01u << UARTOUT_RX_STS_BREAK_SHIFT)
    #define UARTOUT_RX_STS_PAR_ERROR        (uint8)(0x01u << UARTOUT_RX_STS_PAR_ERROR_SHIFT)
    #define UARTOUT_RX_STS_STOP_ERROR       (uint8)(0x01u << UARTOUT_RX_STS_STOP_ERROR_SHIFT)
    #define UARTOUT_RX_STS_OVERRUN          (uint8)(0x01u << UARTOUT_RX_STS_OVERRUN_SHIFT)
    #define UARTOUT_RX_STS_FIFO_NOTEMPTY    (uint8)(0x01u << UARTOUT_RX_STS_FIFO_NOTEMPTY_SHIFT)
    #define UARTOUT_RX_STS_ADDR_MATCH       (uint8)(0x01u << UARTOUT_RX_STS_ADDR_MATCH_SHIFT)
    #define UARTOUT_RX_STS_SOFT_BUFF_OVER   (uint8)(0x01u << UARTOUT_RX_STS_SOFT_BUFF_OVER_SHIFT)
    #define UARTOUT_RX_HW_MASK                     (0x7Fu)
#endif /* End (UARTOUT_RX_ENABLED) || (UARTOUT_HD_ENABLED) */

/* Control Register definitions */
#define UARTOUT_CTRL_HD_SEND_SHIFT                 (0x00u) /* 1 enable TX part in Half Duplex mode */
#define UARTOUT_CTRL_HD_SEND_BREAK_SHIFT           (0x01u) /* 1 send BREAK signal in Half Duplez mode */
#define UARTOUT_CTRL_MARK_SHIFT                    (0x02u) /* 1 sets mark, 0 sets space */
#define UARTOUT_CTRL_PARITY_TYPE0_SHIFT            (0x03u) /* Defines the type of parity implemented */
#define UARTOUT_CTRL_PARITY_TYPE1_SHIFT            (0x04u) /* Defines the type of parity implemented */
#define UARTOUT_CTRL_RXADDR_MODE0_SHIFT            (0x05u)
#define UARTOUT_CTRL_RXADDR_MODE1_SHIFT            (0x06u)
#define UARTOUT_CTRL_RXADDR_MODE2_SHIFT            (0x07u)

#define UARTOUT_CTRL_HD_SEND               (uint8)(0x01u << UARTOUT_CTRL_HD_SEND_SHIFT)
#define UARTOUT_CTRL_HD_SEND_BREAK         (uint8)(0x01u << UARTOUT_CTRL_HD_SEND_BREAK_SHIFT)
#define UARTOUT_CTRL_MARK                  (uint8)(0x01u << UARTOUT_CTRL_MARK_SHIFT)
#define UARTOUT_CTRL_PARITY_TYPE_MASK      (uint8)(0x03u << UARTOUT_CTRL_PARITY_TYPE0_SHIFT)
#define UARTOUT_CTRL_RXADDR_MODE_MASK      (uint8)(0x07u << UARTOUT_CTRL_RXADDR_MODE0_SHIFT)

/* StatusI Register Interrupt Enable Control Bits. As defined by the Register map for the AUX Control Register */
#define UARTOUT_INT_ENABLE                         (0x10u)

/* Bit Counter (7-bit) Control Register Bit Definitions. As defined by the Register map for the AUX Control Register */
#define UARTOUT_CNTR_ENABLE                        (0x20u)

/*   Constants for SendBreak() "retMode" parameter  */
#define UARTOUT_SEND_BREAK                         (0x00u)
#define UARTOUT_WAIT_FOR_COMPLETE_REINIT           (0x01u)
#define UARTOUT_REINIT                             (0x02u)
#define UARTOUT_SEND_WAIT_REINIT                   (0x03u)

#define UARTOUT_OVER_SAMPLE_8                      (8u)
#define UARTOUT_OVER_SAMPLE_16                     (16u)

#define UARTOUT_BIT_CENTER                         (UARTOUT_OVER_SAMPLE_COUNT - 2u)

#define UARTOUT_FIFO_LENGTH                        (4u)
#define UARTOUT_NUMBER_OF_START_BIT                (1u)
#define UARTOUT_MAX_BYTE_VALUE                     (0xFFu)

/* 8X always for count7 implementation */
#define UARTOUT_TXBITCTR_BREAKBITS8X   ((UARTOUT_BREAK_BITS_TX * UARTOUT_OVER_SAMPLE_8) - 1u)
/* 8X or 16X for DP implementation */
#define UARTOUT_TXBITCTR_BREAKBITS ((UARTOUT_BREAK_BITS_TX * UARTOUT_OVER_SAMPLE_COUNT) - 1u)

#define UARTOUT_HALF_BIT_COUNT   \
                            (((UARTOUT_OVER_SAMPLE_COUNT / 2u) + (UARTOUT_USE23POLLING * 1u)) - 2u)
#if (UARTOUT_OVER_SAMPLE_COUNT == UARTOUT_OVER_SAMPLE_8)
    #define UARTOUT_HD_TXBITCTR_INIT   (((UARTOUT_BREAK_BITS_TX + \
                            UARTOUT_NUMBER_OF_START_BIT) * UARTOUT_OVER_SAMPLE_COUNT) - 1u)

    /* This parameter is increased on the 2 in 2 out of 3 mode to sample voting in the middle */
    #define UARTOUT_RXBITCTR_INIT  ((((UARTOUT_BREAK_BITS_RX + UARTOUT_NUMBER_OF_START_BIT) \
                            * UARTOUT_OVER_SAMPLE_COUNT) + UARTOUT_HALF_BIT_COUNT) - 1u)

#else /* UARTOUT_OVER_SAMPLE_COUNT == UARTOUT_OVER_SAMPLE_16 */
    #define UARTOUT_HD_TXBITCTR_INIT   ((8u * UARTOUT_OVER_SAMPLE_COUNT) - 1u)
    /* 7bit counter need one more bit for OverSampleCount = 16 */
    #define UARTOUT_RXBITCTR_INIT      (((7u * UARTOUT_OVER_SAMPLE_COUNT) - 1u) + \
                                                      UARTOUT_HALF_BIT_COUNT)
#endif /* End UARTOUT_OVER_SAMPLE_COUNT */

#define UARTOUT_HD_RXBITCTR_INIT                   UARTOUT_RXBITCTR_INIT


/***************************************
* Global variables external identifier
***************************************/

extern uint8 UARTOUT_initVar;
#if (UARTOUT_TX_INTERRUPT_ENABLED && UARTOUT_TX_ENABLED)
    extern volatile uint8 UARTOUT_txBuffer[UARTOUT_TX_BUFFER_SIZE];
    extern volatile uint8 UARTOUT_txBufferRead;
    extern uint8 UARTOUT_txBufferWrite;
#endif /* (UARTOUT_TX_INTERRUPT_ENABLED && UARTOUT_TX_ENABLED) */
#if (UARTOUT_RX_INTERRUPT_ENABLED && (UARTOUT_RX_ENABLED || UARTOUT_HD_ENABLED))
    extern uint8 UARTOUT_errorStatus;
    extern volatile uint8 UARTOUT_rxBuffer[UARTOUT_RX_BUFFER_SIZE];
    extern volatile uint8 UARTOUT_rxBufferRead;
    extern volatile uint8 UARTOUT_rxBufferWrite;
    extern volatile uint8 UARTOUT_rxBufferLoopDetect;
    extern volatile uint8 UARTOUT_rxBufferOverflow;
    #if (UARTOUT_RXHW_ADDRESS_ENABLED)
        extern volatile uint8 UARTOUT_rxAddressMode;
        extern volatile uint8 UARTOUT_rxAddressDetected;
    #endif /* (UARTOUT_RXHW_ADDRESS_ENABLED) */
#endif /* (UARTOUT_RX_INTERRUPT_ENABLED && (UARTOUT_RX_ENABLED || UARTOUT_HD_ENABLED)) */


/***************************************
* Enumerated Types and Parameters
***************************************/

#define UARTOUT__B_UART__AM_SW_BYTE_BYTE 1
#define UARTOUT__B_UART__AM_SW_DETECT_TO_BUFFER 2
#define UARTOUT__B_UART__AM_HW_BYTE_BY_BYTE 3
#define UARTOUT__B_UART__AM_HW_DETECT_TO_BUFFER 4
#define UARTOUT__B_UART__AM_NONE 0

#define UARTOUT__B_UART__NONE_REVB 0
#define UARTOUT__B_UART__EVEN_REVB 1
#define UARTOUT__B_UART__ODD_REVB 2
#define UARTOUT__B_UART__MARK_SPACE_REVB 3



/***************************************
*    Initial Parameter Constants
***************************************/

/* UART shifts max 8 bits, Mark/Space functionality working if 9 selected */
#define UARTOUT_NUMBER_OF_DATA_BITS    ((8u > 8u) ? 8u : 8u)
#define UARTOUT_NUMBER_OF_STOP_BITS    (2u)

#if (UARTOUT_RXHW_ADDRESS_ENABLED)
    #define UARTOUT_RX_ADDRESS_MODE    (0u)
    #define UARTOUT_RX_HW_ADDRESS1     (0u)
    #define UARTOUT_RX_HW_ADDRESS2     (0u)
#endif /* (UARTOUT_RXHW_ADDRESS_ENABLED) */

#define UARTOUT_INIT_RX_INTERRUPTS_MASK \
                                  (uint8)((0 << UARTOUT_RX_STS_FIFO_NOTEMPTY_SHIFT) \
                                        | (0 << UARTOUT_RX_STS_MRKSPC_SHIFT) \
                                        | (0 << UARTOUT_RX_STS_ADDR_MATCH_SHIFT) \
                                        | (0 << UARTOUT_RX_STS_PAR_ERROR_SHIFT) \
                                        | (0 << UARTOUT_RX_STS_STOP_ERROR_SHIFT) \
                                        | (0 << UARTOUT_RX_STS_BREAK_SHIFT) \
                                        | (0 << UARTOUT_RX_STS_OVERRUN_SHIFT))

#define UARTOUT_INIT_TX_INTERRUPTS_MASK \
                                  (uint8)((1 << UARTOUT_TX_STS_COMPLETE_SHIFT) \
                                        | (0 << UARTOUT_TX_STS_FIFO_EMPTY_SHIFT) \
                                        | (0 << UARTOUT_TX_STS_FIFO_FULL_SHIFT) \
                                        | (0 << UARTOUT_TX_STS_FIFO_NOT_FULL_SHIFT))


/***************************************
*              Registers
***************************************/

#ifdef UARTOUT_BUART_sCR_SyncCtl_CtrlReg__CONTROL_REG
    #define UARTOUT_CONTROL_REG \
                            (* (reg8 *) UARTOUT_BUART_sCR_SyncCtl_CtrlReg__CONTROL_REG )
    #define UARTOUT_CONTROL_PTR \
                            (  (reg8 *) UARTOUT_BUART_sCR_SyncCtl_CtrlReg__CONTROL_REG )
#endif /* End UARTOUT_BUART_sCR_SyncCtl_CtrlReg__CONTROL_REG */

#if(UARTOUT_TX_ENABLED)
    #define UARTOUT_TXDATA_REG          (* (reg8 *) UARTOUT_BUART_sTX_TxShifter_u0__F0_REG)
    #define UARTOUT_TXDATA_PTR          (  (reg8 *) UARTOUT_BUART_sTX_TxShifter_u0__F0_REG)
    #define UARTOUT_TXDATA_AUX_CTL_REG  (* (reg8 *) UARTOUT_BUART_sTX_TxShifter_u0__DP_AUX_CTL_REG)
    #define UARTOUT_TXDATA_AUX_CTL_PTR  (  (reg8 *) UARTOUT_BUART_sTX_TxShifter_u0__DP_AUX_CTL_REG)
    #define UARTOUT_TXSTATUS_REG        (* (reg8 *) UARTOUT_BUART_sTX_TxSts__STATUS_REG)
    #define UARTOUT_TXSTATUS_PTR        (  (reg8 *) UARTOUT_BUART_sTX_TxSts__STATUS_REG)
    #define UARTOUT_TXSTATUS_MASK_REG   (* (reg8 *) UARTOUT_BUART_sTX_TxSts__MASK_REG)
    #define UARTOUT_TXSTATUS_MASK_PTR   (  (reg8 *) UARTOUT_BUART_sTX_TxSts__MASK_REG)
    #define UARTOUT_TXSTATUS_ACTL_REG   (* (reg8 *) UARTOUT_BUART_sTX_TxSts__STATUS_AUX_CTL_REG)
    #define UARTOUT_TXSTATUS_ACTL_PTR   (  (reg8 *) UARTOUT_BUART_sTX_TxSts__STATUS_AUX_CTL_REG)

    /* DP clock */
    #if(UARTOUT_TXCLKGEN_DP)
        #define UARTOUT_TXBITCLKGEN_CTR_REG        \
                                        (* (reg8 *) UARTOUT_BUART_sTX_sCLOCK_TxBitClkGen__D0_REG)
        #define UARTOUT_TXBITCLKGEN_CTR_PTR        \
                                        (  (reg8 *) UARTOUT_BUART_sTX_sCLOCK_TxBitClkGen__D0_REG)
        #define UARTOUT_TXBITCLKTX_COMPLETE_REG    \
                                        (* (reg8 *) UARTOUT_BUART_sTX_sCLOCK_TxBitClkGen__D1_REG)
        #define UARTOUT_TXBITCLKTX_COMPLETE_PTR    \
                                        (  (reg8 *) UARTOUT_BUART_sTX_sCLOCK_TxBitClkGen__D1_REG)
    #else     /* Count7 clock*/
        #define UARTOUT_TXBITCTR_PERIOD_REG    \
                                        (* (reg8 *) UARTOUT_BUART_sTX_sCLOCK_TxBitCounter__PERIOD_REG)
        #define UARTOUT_TXBITCTR_PERIOD_PTR    \
                                        (  (reg8 *) UARTOUT_BUART_sTX_sCLOCK_TxBitCounter__PERIOD_REG)
        #define UARTOUT_TXBITCTR_CONTROL_REG   \
                                        (* (reg8 *) UARTOUT_BUART_sTX_sCLOCK_TxBitCounter__CONTROL_AUX_CTL_REG)
        #define UARTOUT_TXBITCTR_CONTROL_PTR   \
                                        (  (reg8 *) UARTOUT_BUART_sTX_sCLOCK_TxBitCounter__CONTROL_AUX_CTL_REG)
        #define UARTOUT_TXBITCTR_COUNTER_REG   \
                                        (* (reg8 *) UARTOUT_BUART_sTX_sCLOCK_TxBitCounter__COUNT_REG)
        #define UARTOUT_TXBITCTR_COUNTER_PTR   \
                                        (  (reg8 *) UARTOUT_BUART_sTX_sCLOCK_TxBitCounter__COUNT_REG)
    #endif /* UARTOUT_TXCLKGEN_DP */

#endif /* End UARTOUT_TX_ENABLED */

#if(UARTOUT_HD_ENABLED)

    #define UARTOUT_TXDATA_REG             (* (reg8 *) UARTOUT_BUART_sRX_RxShifter_u0__F1_REG )
    #define UARTOUT_TXDATA_PTR             (  (reg8 *) UARTOUT_BUART_sRX_RxShifter_u0__F1_REG )
    #define UARTOUT_TXDATA_AUX_CTL_REG     (* (reg8 *) UARTOUT_BUART_sRX_RxShifter_u0__DP_AUX_CTL_REG)
    #define UARTOUT_TXDATA_AUX_CTL_PTR     (  (reg8 *) UARTOUT_BUART_sRX_RxShifter_u0__DP_AUX_CTL_REG)

    #define UARTOUT_TXSTATUS_REG           (* (reg8 *) UARTOUT_BUART_sRX_RxSts__STATUS_REG )
    #define UARTOUT_TXSTATUS_PTR           (  (reg8 *) UARTOUT_BUART_sRX_RxSts__STATUS_REG )
    #define UARTOUT_TXSTATUS_MASK_REG      (* (reg8 *) UARTOUT_BUART_sRX_RxSts__MASK_REG )
    #define UARTOUT_TXSTATUS_MASK_PTR      (  (reg8 *) UARTOUT_BUART_sRX_RxSts__MASK_REG )
    #define UARTOUT_TXSTATUS_ACTL_REG      (* (reg8 *) UARTOUT_BUART_sRX_RxSts__STATUS_AUX_CTL_REG )
    #define UARTOUT_TXSTATUS_ACTL_PTR      (  (reg8 *) UARTOUT_BUART_sRX_RxSts__STATUS_AUX_CTL_REG )
#endif /* End UARTOUT_HD_ENABLED */

#if( (UARTOUT_RX_ENABLED) || (UARTOUT_HD_ENABLED) )
    #define UARTOUT_RXDATA_REG             (* (reg8 *) UARTOUT_BUART_sRX_RxShifter_u0__F0_REG )
    #define UARTOUT_RXDATA_PTR             (  (reg8 *) UARTOUT_BUART_sRX_RxShifter_u0__F0_REG )
    #define UARTOUT_RXADDRESS1_REG         (* (reg8 *) UARTOUT_BUART_sRX_RxShifter_u0__D0_REG )
    #define UARTOUT_RXADDRESS1_PTR         (  (reg8 *) UARTOUT_BUART_sRX_RxShifter_u0__D0_REG )
    #define UARTOUT_RXADDRESS2_REG         (* (reg8 *) UARTOUT_BUART_sRX_RxShifter_u0__D1_REG )
    #define UARTOUT_RXADDRESS2_PTR         (  (reg8 *) UARTOUT_BUART_sRX_RxShifter_u0__D1_REG )
    #define UARTOUT_RXDATA_AUX_CTL_REG     (* (reg8 *) UARTOUT_BUART_sRX_RxShifter_u0__DP_AUX_CTL_REG)

    #define UARTOUT_RXBITCTR_PERIOD_REG    (* (reg8 *) UARTOUT_BUART_sRX_RxBitCounter__PERIOD_REG )
    #define UARTOUT_RXBITCTR_PERIOD_PTR    (  (reg8 *) UARTOUT_BUART_sRX_RxBitCounter__PERIOD_REG )
    #define UARTOUT_RXBITCTR_CONTROL_REG   \
                                        (* (reg8 *) UARTOUT_BUART_sRX_RxBitCounter__CONTROL_AUX_CTL_REG )
    #define UARTOUT_RXBITCTR_CONTROL_PTR   \
                                        (  (reg8 *) UARTOUT_BUART_sRX_RxBitCounter__CONTROL_AUX_CTL_REG )
    #define UARTOUT_RXBITCTR_COUNTER_REG   (* (reg8 *) UARTOUT_BUART_sRX_RxBitCounter__COUNT_REG )
    #define UARTOUT_RXBITCTR_COUNTER_PTR   (  (reg8 *) UARTOUT_BUART_sRX_RxBitCounter__COUNT_REG )

    #define UARTOUT_RXSTATUS_REG           (* (reg8 *) UARTOUT_BUART_sRX_RxSts__STATUS_REG )
    #define UARTOUT_RXSTATUS_PTR           (  (reg8 *) UARTOUT_BUART_sRX_RxSts__STATUS_REG )
    #define UARTOUT_RXSTATUS_MASK_REG      (* (reg8 *) UARTOUT_BUART_sRX_RxSts__MASK_REG )
    #define UARTOUT_RXSTATUS_MASK_PTR      (  (reg8 *) UARTOUT_BUART_sRX_RxSts__MASK_REG )
    #define UARTOUT_RXSTATUS_ACTL_REG      (* (reg8 *) UARTOUT_BUART_sRX_RxSts__STATUS_AUX_CTL_REG )
    #define UARTOUT_RXSTATUS_ACTL_PTR      (  (reg8 *) UARTOUT_BUART_sRX_RxSts__STATUS_AUX_CTL_REG )
#endif /* End  (UARTOUT_RX_ENABLED) || (UARTOUT_HD_ENABLED) */

#if(UARTOUT_INTERNAL_CLOCK_USED)
    /* Register to enable or disable the digital clocks */
    #define UARTOUT_INTCLOCK_CLKEN_REG     (* (reg8 *) UARTOUT_IntClock__PM_ACT_CFG)
    #define UARTOUT_INTCLOCK_CLKEN_PTR     (  (reg8 *) UARTOUT_IntClock__PM_ACT_CFG)

    /* Clock mask for this clock. */
    #define UARTOUT_INTCLOCK_CLKEN_MASK    UARTOUT_IntClock__PM_ACT_MSK
#endif /* End UARTOUT_INTERNAL_CLOCK_USED */


/***************************************
*       Register Constants
***************************************/

#if(UARTOUT_TX_ENABLED)
    #define UARTOUT_TX_FIFO_CLR            (0x01u) /* FIFO0 CLR */
#endif /* End UARTOUT_TX_ENABLED */

#if(UARTOUT_HD_ENABLED)
    #define UARTOUT_TX_FIFO_CLR            (0x02u) /* FIFO1 CLR */
#endif /* End UARTOUT_HD_ENABLED */

#if( (UARTOUT_RX_ENABLED) || (UARTOUT_HD_ENABLED) )
    #define UARTOUT_RX_FIFO_CLR            (0x01u) /* FIFO0 CLR */
#endif /* End  (UARTOUT_RX_ENABLED) || (UARTOUT_HD_ENABLED) */


/***************************************
* The following code is DEPRECATED and
* should not be used in new projects.
***************************************/

/* UART v2_40 obsolete definitions */
#define UARTOUT_WAIT_1_MS      UARTOUT_BL_CHK_DELAY_MS   

#define UARTOUT_TXBUFFERSIZE   UARTOUT_TX_BUFFER_SIZE
#define UARTOUT_RXBUFFERSIZE   UARTOUT_RX_BUFFER_SIZE

#if (UARTOUT_RXHW_ADDRESS_ENABLED)
    #define UARTOUT_RXADDRESSMODE  UARTOUT_RX_ADDRESS_MODE
    #define UARTOUT_RXHWADDRESS1   UARTOUT_RX_HW_ADDRESS1
    #define UARTOUT_RXHWADDRESS2   UARTOUT_RX_HW_ADDRESS2
    /* Backward compatible define */
    #define UARTOUT_RXAddressMode  UARTOUT_RXADDRESSMODE
#endif /* (UARTOUT_RXHW_ADDRESS_ENABLED) */

/* UART v2_30 obsolete definitions */
#define UARTOUT_initvar                    UARTOUT_initVar

#define UARTOUT_RX_Enabled                 UARTOUT_RX_ENABLED
#define UARTOUT_TX_Enabled                 UARTOUT_TX_ENABLED
#define UARTOUT_HD_Enabled                 UARTOUT_HD_ENABLED
#define UARTOUT_RX_IntInterruptEnabled     UARTOUT_RX_INTERRUPT_ENABLED
#define UARTOUT_TX_IntInterruptEnabled     UARTOUT_TX_INTERRUPT_ENABLED
#define UARTOUT_InternalClockUsed          UARTOUT_INTERNAL_CLOCK_USED
#define UARTOUT_RXHW_Address_Enabled       UARTOUT_RXHW_ADDRESS_ENABLED
#define UARTOUT_OverSampleCount            UARTOUT_OVER_SAMPLE_COUNT
#define UARTOUT_ParityType                 UARTOUT_PARITY_TYPE

#if( UARTOUT_TX_ENABLED && (UARTOUT_TXBUFFERSIZE > UARTOUT_FIFO_LENGTH))
    #define UARTOUT_TXBUFFER               UARTOUT_txBuffer
    #define UARTOUT_TXBUFFERREAD           UARTOUT_txBufferRead
    #define UARTOUT_TXBUFFERWRITE          UARTOUT_txBufferWrite
#endif /* End UARTOUT_TX_ENABLED */
#if( ( UARTOUT_RX_ENABLED || UARTOUT_HD_ENABLED ) && \
     (UARTOUT_RXBUFFERSIZE > UARTOUT_FIFO_LENGTH) )
    #define UARTOUT_RXBUFFER               UARTOUT_rxBuffer
    #define UARTOUT_RXBUFFERREAD           UARTOUT_rxBufferRead
    #define UARTOUT_RXBUFFERWRITE          UARTOUT_rxBufferWrite
    #define UARTOUT_RXBUFFERLOOPDETECT     UARTOUT_rxBufferLoopDetect
    #define UARTOUT_RXBUFFER_OVERFLOW      UARTOUT_rxBufferOverflow
#endif /* End UARTOUT_RX_ENABLED */

#ifdef UARTOUT_BUART_sCR_SyncCtl_CtrlReg__CONTROL_REG
    #define UARTOUT_CONTROL                UARTOUT_CONTROL_REG
#endif /* End UARTOUT_BUART_sCR_SyncCtl_CtrlReg__CONTROL_REG */

#if(UARTOUT_TX_ENABLED)
    #define UARTOUT_TXDATA                 UARTOUT_TXDATA_REG
    #define UARTOUT_TXSTATUS               UARTOUT_TXSTATUS_REG
    #define UARTOUT_TXSTATUS_MASK          UARTOUT_TXSTATUS_MASK_REG
    #define UARTOUT_TXSTATUS_ACTL          UARTOUT_TXSTATUS_ACTL_REG
    /* DP clock */
    #if(UARTOUT_TXCLKGEN_DP)
        #define UARTOUT_TXBITCLKGEN_CTR        UARTOUT_TXBITCLKGEN_CTR_REG
        #define UARTOUT_TXBITCLKTX_COMPLETE    UARTOUT_TXBITCLKTX_COMPLETE_REG
    #else     /* Count7 clock*/
        #define UARTOUT_TXBITCTR_PERIOD        UARTOUT_TXBITCTR_PERIOD_REG
        #define UARTOUT_TXBITCTR_CONTROL       UARTOUT_TXBITCTR_CONTROL_REG
        #define UARTOUT_TXBITCTR_COUNTER       UARTOUT_TXBITCTR_COUNTER_REG
    #endif /* UARTOUT_TXCLKGEN_DP */
#endif /* End UARTOUT_TX_ENABLED */

#if(UARTOUT_HD_ENABLED)
    #define UARTOUT_TXDATA                 UARTOUT_TXDATA_REG
    #define UARTOUT_TXSTATUS               UARTOUT_TXSTATUS_REG
    #define UARTOUT_TXSTATUS_MASK          UARTOUT_TXSTATUS_MASK_REG
    #define UARTOUT_TXSTATUS_ACTL          UARTOUT_TXSTATUS_ACTL_REG
#endif /* End UARTOUT_HD_ENABLED */

#if( (UARTOUT_RX_ENABLED) || (UARTOUT_HD_ENABLED) )
    #define UARTOUT_RXDATA                 UARTOUT_RXDATA_REG
    #define UARTOUT_RXADDRESS1             UARTOUT_RXADDRESS1_REG
    #define UARTOUT_RXADDRESS2             UARTOUT_RXADDRESS2_REG
    #define UARTOUT_RXBITCTR_PERIOD        UARTOUT_RXBITCTR_PERIOD_REG
    #define UARTOUT_RXBITCTR_CONTROL       UARTOUT_RXBITCTR_CONTROL_REG
    #define UARTOUT_RXBITCTR_COUNTER       UARTOUT_RXBITCTR_COUNTER_REG
    #define UARTOUT_RXSTATUS               UARTOUT_RXSTATUS_REG
    #define UARTOUT_RXSTATUS_MASK          UARTOUT_RXSTATUS_MASK_REG
    #define UARTOUT_RXSTATUS_ACTL          UARTOUT_RXSTATUS_ACTL_REG
#endif /* End  (UARTOUT_RX_ENABLED) || (UARTOUT_HD_ENABLED) */

#if(UARTOUT_INTERNAL_CLOCK_USED)
    #define UARTOUT_INTCLOCK_CLKEN         UARTOUT_INTCLOCK_CLKEN_REG
#endif /* End UARTOUT_INTERNAL_CLOCK_USED */

#define UARTOUT_WAIT_FOR_COMLETE_REINIT    UARTOUT_WAIT_FOR_COMPLETE_REINIT

#endif  /* CY_UART_UARTOUT_H */


/* [] END OF FILE */
