/*******************************************************************************
* File Name: UARTOUT.c
* Version 2.50
*
* Description:
*  This file provides all API functionality of the UART component
*
* Note:
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "UARTOUT.h"
#if (UARTOUT_INTERNAL_CLOCK_USED)
    #include "UARTOUT_IntClock.h"
#endif /* End UARTOUT_INTERNAL_CLOCK_USED */


/***************************************
* Global data allocation
***************************************/

uint8 UARTOUT_initVar = 0u;

#if (UARTOUT_TX_INTERRUPT_ENABLED && UARTOUT_TX_ENABLED)
    volatile uint8 UARTOUT_txBuffer[UARTOUT_TX_BUFFER_SIZE];
    volatile uint8 UARTOUT_txBufferRead = 0u;
    uint8 UARTOUT_txBufferWrite = 0u;
#endif /* (UARTOUT_TX_INTERRUPT_ENABLED && UARTOUT_TX_ENABLED) */

#if (UARTOUT_RX_INTERRUPT_ENABLED && (UARTOUT_RX_ENABLED || UARTOUT_HD_ENABLED))
    uint8 UARTOUT_errorStatus = 0u;
    volatile uint8 UARTOUT_rxBuffer[UARTOUT_RX_BUFFER_SIZE];
    volatile uint8 UARTOUT_rxBufferRead  = 0u;
    volatile uint8 UARTOUT_rxBufferWrite = 0u;
    volatile uint8 UARTOUT_rxBufferLoopDetect = 0u;
    volatile uint8 UARTOUT_rxBufferOverflow   = 0u;
    #if (UARTOUT_RXHW_ADDRESS_ENABLED)
        volatile uint8 UARTOUT_rxAddressMode = UARTOUT_RX_ADDRESS_MODE;
        volatile uint8 UARTOUT_rxAddressDetected = 0u;
    #endif /* (UARTOUT_RXHW_ADDRESS_ENABLED) */
#endif /* (UARTOUT_RX_INTERRUPT_ENABLED && (UARTOUT_RX_ENABLED || UARTOUT_HD_ENABLED)) */


/*******************************************************************************
* Function Name: UARTOUT_Start
********************************************************************************
*
* Summary:
*  This is the preferred method to begin component operation.
*  UARTOUT_Start() sets the initVar variable, calls the
*  UARTOUT_Init() function, and then calls the
*  UARTOUT_Enable() function.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  The UARTOUT_intiVar variable is used to indicate initial
*  configuration of this component. The variable is initialized to zero (0u)
*  and set to one (1u) the first time UARTOUT_Start() is called. This
*  allows for component initialization without re-initialization in all
*  subsequent calls to the UARTOUT_Start() routine.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void UARTOUT_Start(void) 
{
    /* If not initialized then initialize all required hardware and software */
    if(UARTOUT_initVar == 0u)
    {
        UARTOUT_Init();
        UARTOUT_initVar = 1u;
    }

    UARTOUT_Enable();
}


/*******************************************************************************
* Function Name: UARTOUT_Init
********************************************************************************
*
* Summary:
*  Initializes or restores the component according to the customizer Configure
*  dialog settings. It is not necessary to call UARTOUT_Init() because
*  the UARTOUT_Start() API calls this function and is the preferred
*  method to begin component operation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void UARTOUT_Init(void) 
{
    #if(UARTOUT_RX_ENABLED || UARTOUT_HD_ENABLED)

        #if (UARTOUT_RX_INTERRUPT_ENABLED)
            /* Set RX interrupt vector and priority */
            (void) CyIntSetVector(UARTOUT_RX_VECT_NUM, &UARTOUT_RXISR);
            CyIntSetPriority(UARTOUT_RX_VECT_NUM, UARTOUT_RX_PRIOR_NUM);
            UARTOUT_errorStatus = 0u;
        #endif /* (UARTOUT_RX_INTERRUPT_ENABLED) */

        #if (UARTOUT_RXHW_ADDRESS_ENABLED)
            UARTOUT_SetRxAddressMode(UARTOUT_RX_ADDRESS_MODE);
            UARTOUT_SetRxAddress1(UARTOUT_RX_HW_ADDRESS1);
            UARTOUT_SetRxAddress2(UARTOUT_RX_HW_ADDRESS2);
        #endif /* End UARTOUT_RXHW_ADDRESS_ENABLED */

        /* Init Count7 period */
        UARTOUT_RXBITCTR_PERIOD_REG = UARTOUT_RXBITCTR_INIT;
        /* Configure the Initial RX interrupt mask */
        UARTOUT_RXSTATUS_MASK_REG  = UARTOUT_INIT_RX_INTERRUPTS_MASK;
    #endif /* End UARTOUT_RX_ENABLED || UARTOUT_HD_ENABLED*/

    #if(UARTOUT_TX_ENABLED)
        #if (UARTOUT_TX_INTERRUPT_ENABLED)
            /* Set TX interrupt vector and priority */
            (void) CyIntSetVector(UARTOUT_TX_VECT_NUM, &UARTOUT_TXISR);
            CyIntSetPriority(UARTOUT_TX_VECT_NUM, UARTOUT_TX_PRIOR_NUM);
        #endif /* (UARTOUT_TX_INTERRUPT_ENABLED) */

        /* Write Counter Value for TX Bit Clk Generator*/
        #if (UARTOUT_TXCLKGEN_DP)
            UARTOUT_TXBITCLKGEN_CTR_REG = UARTOUT_BIT_CENTER;
            UARTOUT_TXBITCLKTX_COMPLETE_REG = ((UARTOUT_NUMBER_OF_DATA_BITS +
                        UARTOUT_NUMBER_OF_START_BIT) * UARTOUT_OVER_SAMPLE_COUNT) - 1u;
        #else
            UARTOUT_TXBITCTR_PERIOD_REG = ((UARTOUT_NUMBER_OF_DATA_BITS +
                        UARTOUT_NUMBER_OF_START_BIT) * UARTOUT_OVER_SAMPLE_8) - 1u;
        #endif /* End UARTOUT_TXCLKGEN_DP */

        /* Configure the Initial TX interrupt mask */
        #if (UARTOUT_TX_INTERRUPT_ENABLED)
            UARTOUT_TXSTATUS_MASK_REG = UARTOUT_TX_STS_FIFO_EMPTY;
        #else
            UARTOUT_TXSTATUS_MASK_REG = UARTOUT_INIT_TX_INTERRUPTS_MASK;
        #endif /*End UARTOUT_TX_INTERRUPT_ENABLED*/

    #endif /* End UARTOUT_TX_ENABLED */

    #if(UARTOUT_PARITY_TYPE_SW)  /* Write Parity to Control Register */
        UARTOUT_WriteControlRegister( \
            (UARTOUT_ReadControlRegister() & (uint8)~UARTOUT_CTRL_PARITY_TYPE_MASK) | \
            (uint8)(UARTOUT_PARITY_TYPE << UARTOUT_CTRL_PARITY_TYPE0_SHIFT) );
    #endif /* End UARTOUT_PARITY_TYPE_SW */
}


/*******************************************************************************
* Function Name: UARTOUT_Enable
********************************************************************************
*
* Summary:
*  Activates the hardware and begins component operation. It is not necessary
*  to call UARTOUT_Enable() because the UARTOUT_Start() API
*  calls this function, which is the preferred method to begin component
*  operation.

* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  UARTOUT_rxAddressDetected - set to initial state (0).
*
*******************************************************************************/
void UARTOUT_Enable(void) 
{
    uint8 enableInterrupts;
    enableInterrupts = CyEnterCriticalSection();

    #if (UARTOUT_RX_ENABLED || UARTOUT_HD_ENABLED)
        /* RX Counter (Count7) Enable */
        UARTOUT_RXBITCTR_CONTROL_REG |= UARTOUT_CNTR_ENABLE;

        /* Enable the RX Interrupt */
        UARTOUT_RXSTATUS_ACTL_REG  |= UARTOUT_INT_ENABLE;

        #if (UARTOUT_RX_INTERRUPT_ENABLED)
            UARTOUT_EnableRxInt();

            #if (UARTOUT_RXHW_ADDRESS_ENABLED)
                UARTOUT_rxAddressDetected = 0u;
            #endif /* (UARTOUT_RXHW_ADDRESS_ENABLED) */
        #endif /* (UARTOUT_RX_INTERRUPT_ENABLED) */
    #endif /* (UARTOUT_RX_ENABLED || UARTOUT_HD_ENABLED) */

    #if(UARTOUT_TX_ENABLED)
        /* TX Counter (DP/Count7) Enable */
        #if(!UARTOUT_TXCLKGEN_DP)
            UARTOUT_TXBITCTR_CONTROL_REG |= UARTOUT_CNTR_ENABLE;
        #endif /* End UARTOUT_TXCLKGEN_DP */

        /* Enable the TX Interrupt */
        UARTOUT_TXSTATUS_ACTL_REG |= UARTOUT_INT_ENABLE;
        #if (UARTOUT_TX_INTERRUPT_ENABLED)
            UARTOUT_ClearPendingTxInt(); /* Clear history of TX_NOT_EMPTY */
            UARTOUT_EnableTxInt();
        #endif /* (UARTOUT_TX_INTERRUPT_ENABLED) */
     #endif /* (UARTOUT_TX_INTERRUPT_ENABLED) */

    #if (UARTOUT_INTERNAL_CLOCK_USED)
        UARTOUT_IntClock_Start();  /* Enable the clock */
    #endif /* (UARTOUT_INTERNAL_CLOCK_USED) */

    CyExitCriticalSection(enableInterrupts);
}


/*******************************************************************************
* Function Name: UARTOUT_Stop
********************************************************************************
*
* Summary:
*  Disables the UART operation.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void UARTOUT_Stop(void) 
{
    uint8 enableInterrupts;
    enableInterrupts = CyEnterCriticalSection();

    /* Write Bit Counter Disable */
    #if (UARTOUT_RX_ENABLED || UARTOUT_HD_ENABLED)
        UARTOUT_RXBITCTR_CONTROL_REG &= (uint8) ~UARTOUT_CNTR_ENABLE;
    #endif /* (UARTOUT_RX_ENABLED || UARTOUT_HD_ENABLED) */

    #if (UARTOUT_TX_ENABLED)
        #if(!UARTOUT_TXCLKGEN_DP)
            UARTOUT_TXBITCTR_CONTROL_REG &= (uint8) ~UARTOUT_CNTR_ENABLE;
        #endif /* (!UARTOUT_TXCLKGEN_DP) */
    #endif /* (UARTOUT_TX_ENABLED) */

    #if (UARTOUT_INTERNAL_CLOCK_USED)
        UARTOUT_IntClock_Stop();   /* Disable the clock */
    #endif /* (UARTOUT_INTERNAL_CLOCK_USED) */

    /* Disable internal interrupt component */
    #if (UARTOUT_RX_ENABLED || UARTOUT_HD_ENABLED)
        UARTOUT_RXSTATUS_ACTL_REG  &= (uint8) ~UARTOUT_INT_ENABLE;

        #if (UARTOUT_RX_INTERRUPT_ENABLED)
            UARTOUT_DisableRxInt();
        #endif /* (UARTOUT_RX_INTERRUPT_ENABLED) */
    #endif /* (UARTOUT_RX_ENABLED || UARTOUT_HD_ENABLED) */

    #if (UARTOUT_TX_ENABLED)
        UARTOUT_TXSTATUS_ACTL_REG &= (uint8) ~UARTOUT_INT_ENABLE;

        #if (UARTOUT_TX_INTERRUPT_ENABLED)
            UARTOUT_DisableTxInt();
        #endif /* (UARTOUT_TX_INTERRUPT_ENABLED) */
    #endif /* (UARTOUT_TX_ENABLED) */

    CyExitCriticalSection(enableInterrupts);
}


/*******************************************************************************
* Function Name: UARTOUT_ReadControlRegister
********************************************************************************
*
* Summary:
*  Returns the current value of the control register.
*
* Parameters:
*  None.
*
* Return:
*  Contents of the control register.
*
*******************************************************************************/
uint8 UARTOUT_ReadControlRegister(void) 
{
    #if (UARTOUT_CONTROL_REG_REMOVED)
        return(0u);
    #else
        return(UARTOUT_CONTROL_REG);
    #endif /* (UARTOUT_CONTROL_REG_REMOVED) */
}


/*******************************************************************************
* Function Name: UARTOUT_WriteControlRegister
********************************************************************************
*
* Summary:
*  Writes an 8-bit value into the control register
*
* Parameters:
*  control:  control register value
*
* Return:
*  None.
*
*******************************************************************************/
void  UARTOUT_WriteControlRegister(uint8 control) 
{
    #if (UARTOUT_CONTROL_REG_REMOVED)
        if(0u != control)
        {
            /* Suppress compiler warning */
        }
    #else
       UARTOUT_CONTROL_REG = control;
    #endif /* (UARTOUT_CONTROL_REG_REMOVED) */
}


#if(UARTOUT_RX_ENABLED || UARTOUT_HD_ENABLED)
    /*******************************************************************************
    * Function Name: UARTOUT_SetRxInterruptMode
    ********************************************************************************
    *
    * Summary:
    *  Configures the RX interrupt sources enabled.
    *
    * Parameters:
    *  IntSrc:  Bit field containing the RX interrupts to enable. Based on the 
    *  bit-field arrangement of the status register. This value must be a 
    *  combination of status register bit-masks shown below:
    *      UARTOUT_RX_STS_FIFO_NOTEMPTY    Interrupt on byte received.
    *      UARTOUT_RX_STS_PAR_ERROR        Interrupt on parity error.
    *      UARTOUT_RX_STS_STOP_ERROR       Interrupt on stop error.
    *      UARTOUT_RX_STS_BREAK            Interrupt on break.
    *      UARTOUT_RX_STS_OVERRUN          Interrupt on overrun error.
    *      UARTOUT_RX_STS_ADDR_MATCH       Interrupt on address match.
    *      UARTOUT_RX_STS_MRKSPC           Interrupt on address detect.
    *
    * Return:
    *  None.
    *
    * Theory:
    *  Enables the output of specific status bits to the interrupt controller
    *
    *******************************************************************************/
    void UARTOUT_SetRxInterruptMode(uint8 intSrc) 
    {
        UARTOUT_RXSTATUS_MASK_REG  = intSrc;
    }


    /*******************************************************************************
    * Function Name: UARTOUT_ReadRxData
    ********************************************************************************
    *
    * Summary:
    *  Returns the next byte of received data. This function returns data without
    *  checking the status. You must check the status separately.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Received data from RX register
    *
    * Global Variables:
    *  UARTOUT_rxBuffer - RAM buffer pointer for save received data.
    *  UARTOUT_rxBufferWrite - cyclic index for write to rxBuffer,
    *     checked to identify new data.
    *  UARTOUT_rxBufferRead - cyclic index for read from rxBuffer,
    *     incremented after each byte has been read from buffer.
    *  UARTOUT_rxBufferLoopDetect - cleared if loop condition was detected
    *     in RX ISR.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    uint8 UARTOUT_ReadRxData(void) 
    {
        uint8 rxData;

    #if (UARTOUT_RX_INTERRUPT_ENABLED)

        uint8 locRxBufferRead;
        uint8 locRxBufferWrite;

        /* Protect variables that could change on interrupt */
        UARTOUT_DisableRxInt();

        locRxBufferRead  = UARTOUT_rxBufferRead;
        locRxBufferWrite = UARTOUT_rxBufferWrite;

        if( (UARTOUT_rxBufferLoopDetect != 0u) || (locRxBufferRead != locRxBufferWrite) )
        {
            rxData = UARTOUT_rxBuffer[locRxBufferRead];
            locRxBufferRead++;

            if(locRxBufferRead >= UARTOUT_RX_BUFFER_SIZE)
            {
                locRxBufferRead = 0u;
            }
            /* Update the real pointer */
            UARTOUT_rxBufferRead = locRxBufferRead;

            if(UARTOUT_rxBufferLoopDetect != 0u)
            {
                UARTOUT_rxBufferLoopDetect = 0u;
                #if ((UARTOUT_RX_INTERRUPT_ENABLED) && (UARTOUT_FLOW_CONTROL != 0u))
                    /* When Hardware Flow Control selected - return RX mask */
                    #if( UARTOUT_HD_ENABLED )
                        if((UARTOUT_CONTROL_REG & UARTOUT_CTRL_HD_SEND) == 0u)
                        {   /* In Half duplex mode return RX mask only in RX
                            *  configuration set, otherwise
                            *  mask will be returned in LoadRxConfig() API.
                            */
                            UARTOUT_RXSTATUS_MASK_REG  |= UARTOUT_RX_STS_FIFO_NOTEMPTY;
                        }
                    #else
                        UARTOUT_RXSTATUS_MASK_REG  |= UARTOUT_RX_STS_FIFO_NOTEMPTY;
                    #endif /* end UARTOUT_HD_ENABLED */
                #endif /* ((UARTOUT_RX_INTERRUPT_ENABLED) && (UARTOUT_FLOW_CONTROL != 0u)) */
            }
        }
        else
        {   /* Needs to check status for RX_STS_FIFO_NOTEMPTY bit */
            rxData = UARTOUT_RXDATA_REG;
        }

        UARTOUT_EnableRxInt();

    #else

        /* Needs to check status for RX_STS_FIFO_NOTEMPTY bit */
        rxData = UARTOUT_RXDATA_REG;

    #endif /* (UARTOUT_RX_INTERRUPT_ENABLED) */

        return(rxData);
    }


    /*******************************************************************************
    * Function Name: UARTOUT_ReadRxStatus
    ********************************************************************************
    *
    * Summary:
    *  Returns the current state of the receiver status register and the software
    *  buffer overflow status.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Current state of the status register.
    *
    * Side Effect:
    *  All status register bits are clear-on-read except
    *  UARTOUT_RX_STS_FIFO_NOTEMPTY.
    *  UARTOUT_RX_STS_FIFO_NOTEMPTY clears immediately after RX data
    *  register read.
    *
    * Global Variables:
    *  UARTOUT_rxBufferOverflow - used to indicate overload condition.
    *   It set to one in RX interrupt when there isn't free space in
    *   UARTOUT_rxBufferRead to write new data. This condition returned
    *   and cleared to zero by this API as an
    *   UARTOUT_RX_STS_SOFT_BUFF_OVER bit along with RX Status register
    *   bits.
    *
    *******************************************************************************/
    uint8 UARTOUT_ReadRxStatus(void) 
    {
        uint8 status;

        status = UARTOUT_RXSTATUS_REG & UARTOUT_RX_HW_MASK;

    #if (UARTOUT_RX_INTERRUPT_ENABLED)
        if(UARTOUT_rxBufferOverflow != 0u)
        {
            status |= UARTOUT_RX_STS_SOFT_BUFF_OVER;
            UARTOUT_rxBufferOverflow = 0u;
        }
    #endif /* (UARTOUT_RX_INTERRUPT_ENABLED) */

        return(status);
    }


    /*******************************************************************************
    * Function Name: UARTOUT_GetChar
    ********************************************************************************
    *
    * Summary:
    *  Returns the last received byte of data. UARTOUT_GetChar() is
    *  designed for ASCII characters and returns a uint8 where 1 to 255 are values
    *  for valid characters and 0 indicates an error occurred or no data is present.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Character read from UART RX buffer. ASCII characters from 1 to 255 are valid.
    *  A returned zero signifies an error condition or no data available.
    *
    * Global Variables:
    *  UARTOUT_rxBuffer - RAM buffer pointer for save received data.
    *  UARTOUT_rxBufferWrite - cyclic index for write to rxBuffer,
    *     checked to identify new data.
    *  UARTOUT_rxBufferRead - cyclic index for read from rxBuffer,
    *     incremented after each byte has been read from buffer.
    *  UARTOUT_rxBufferLoopDetect - cleared if loop condition was detected
    *     in RX ISR.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    uint8 UARTOUT_GetChar(void) 
    {
        uint8 rxData = 0u;
        uint8 rxStatus;

    #if (UARTOUT_RX_INTERRUPT_ENABLED)
        uint8 locRxBufferRead;
        uint8 locRxBufferWrite;

        /* Protect variables that could change on interrupt */
        UARTOUT_DisableRxInt();

        locRxBufferRead  = UARTOUT_rxBufferRead;
        locRxBufferWrite = UARTOUT_rxBufferWrite;

        if( (UARTOUT_rxBufferLoopDetect != 0u) || (locRxBufferRead != locRxBufferWrite) )
        {
            rxData = UARTOUT_rxBuffer[locRxBufferRead];
            locRxBufferRead++;
            if(locRxBufferRead >= UARTOUT_RX_BUFFER_SIZE)
            {
                locRxBufferRead = 0u;
            }
            /* Update the real pointer */
            UARTOUT_rxBufferRead = locRxBufferRead;

            if(UARTOUT_rxBufferLoopDetect != 0u)
            {
                UARTOUT_rxBufferLoopDetect = 0u;
                #if( (UARTOUT_RX_INTERRUPT_ENABLED) && (UARTOUT_FLOW_CONTROL != 0u) )
                    /* When Hardware Flow Control selected - return RX mask */
                    #if( UARTOUT_HD_ENABLED )
                        if((UARTOUT_CONTROL_REG & UARTOUT_CTRL_HD_SEND) == 0u)
                        {   /* In Half duplex mode return RX mask only if
                            *  RX configuration set, otherwise
                            *  mask will be returned in LoadRxConfig() API.
                            */
                            UARTOUT_RXSTATUS_MASK_REG |= UARTOUT_RX_STS_FIFO_NOTEMPTY;
                        }
                    #else
                        UARTOUT_RXSTATUS_MASK_REG |= UARTOUT_RX_STS_FIFO_NOTEMPTY;
                    #endif /* end UARTOUT_HD_ENABLED */
                #endif /* UARTOUT_RX_INTERRUPT_ENABLED and Hardware flow control*/
            }

        }
        else
        {   rxStatus = UARTOUT_RXSTATUS_REG;
            if((rxStatus & UARTOUT_RX_STS_FIFO_NOTEMPTY) != 0u)
            {   /* Read received data from FIFO */
                rxData = UARTOUT_RXDATA_REG;
                /*Check status on error*/
                if((rxStatus & (UARTOUT_RX_STS_BREAK | UARTOUT_RX_STS_PAR_ERROR |
                                UARTOUT_RX_STS_STOP_ERROR | UARTOUT_RX_STS_OVERRUN)) != 0u)
                {
                    rxData = 0u;
                }
            }
        }

        UARTOUT_EnableRxInt();

    #else

        rxStatus =UARTOUT_RXSTATUS_REG;
        if((rxStatus & UARTOUT_RX_STS_FIFO_NOTEMPTY) != 0u)
        {
            /* Read received data from FIFO */
            rxData = UARTOUT_RXDATA_REG;

            /*Check status on error*/
            if((rxStatus & (UARTOUT_RX_STS_BREAK | UARTOUT_RX_STS_PAR_ERROR |
                            UARTOUT_RX_STS_STOP_ERROR | UARTOUT_RX_STS_OVERRUN)) != 0u)
            {
                rxData = 0u;
            }
        }
    #endif /* (UARTOUT_RX_INTERRUPT_ENABLED) */

        return(rxData);
    }


    /*******************************************************************************
    * Function Name: UARTOUT_GetByte
    ********************************************************************************
    *
    * Summary:
    *  Reads UART RX buffer immediately, returns received character and error
    *  condition.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  MSB contains status and LSB contains UART RX data. If the MSB is nonzero,
    *  an error has occurred.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    uint16 UARTOUT_GetByte(void) 
    {
        
    #if (UARTOUT_RX_INTERRUPT_ENABLED)
        uint16 locErrorStatus;
        /* Protect variables that could change on interrupt */
        UARTOUT_DisableRxInt();
        locErrorStatus = (uint16)UARTOUT_errorStatus;
        UARTOUT_errorStatus = 0u;
        UARTOUT_EnableRxInt();
        return ( (uint16)(locErrorStatus << 8u) | UARTOUT_ReadRxData() );
    #else
        return ( ((uint16)UARTOUT_ReadRxStatus() << 8u) | UARTOUT_ReadRxData() );
    #endif /* UARTOUT_RX_INTERRUPT_ENABLED */
        
    }


    /*******************************************************************************
    * Function Name: UARTOUT_GetRxBufferSize
    ********************************************************************************
    *
    * Summary:
    *  Returns the number of received bytes available in the RX buffer.
    *  * RX software buffer is disabled (RX Buffer Size parameter is equal to 4): 
    *    returns 0 for empty RX FIFO or 1 for not empty RX FIFO.
    *  * RX software buffer is enabled: returns the number of bytes available in 
    *    the RX software buffer. Bytes available in the RX FIFO do not take to 
    *    account.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  uint8: Number of bytes in the RX buffer. 
    *    Return value type depends on RX Buffer Size parameter.
    *
    * Global Variables:
    *  UARTOUT_rxBufferWrite - used to calculate left bytes.
    *  UARTOUT_rxBufferRead - used to calculate left bytes.
    *  UARTOUT_rxBufferLoopDetect - checked to decide left bytes amount.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Allows the user to find out how full the RX Buffer is.
    *
    *******************************************************************************/
    uint8 UARTOUT_GetRxBufferSize(void)
                                                            
    {
        uint8 size;

    #if (UARTOUT_RX_INTERRUPT_ENABLED)

        /* Protect variables that could change on interrupt */
        UARTOUT_DisableRxInt();

        if(UARTOUT_rxBufferRead == UARTOUT_rxBufferWrite)
        {
            if(UARTOUT_rxBufferLoopDetect != 0u)
            {
                size = UARTOUT_RX_BUFFER_SIZE;
            }
            else
            {
                size = 0u;
            }
        }
        else if(UARTOUT_rxBufferRead < UARTOUT_rxBufferWrite)
        {
            size = (UARTOUT_rxBufferWrite - UARTOUT_rxBufferRead);
        }
        else
        {
            size = (UARTOUT_RX_BUFFER_SIZE - UARTOUT_rxBufferRead) + UARTOUT_rxBufferWrite;
        }

        UARTOUT_EnableRxInt();

    #else

        /* We can only know if there is data in the fifo. */
        size = ((UARTOUT_RXSTATUS_REG & UARTOUT_RX_STS_FIFO_NOTEMPTY) != 0u) ? 1u : 0u;

    #endif /* (UARTOUT_RX_INTERRUPT_ENABLED) */

        return(size);
    }


    /*******************************************************************************
    * Function Name: UARTOUT_ClearRxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Clears the receiver memory buffer and hardware RX FIFO of all received data.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  UARTOUT_rxBufferWrite - cleared to zero.
    *  UARTOUT_rxBufferRead - cleared to zero.
    *  UARTOUT_rxBufferLoopDetect - cleared to zero.
    *  UARTOUT_rxBufferOverflow - cleared to zero.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Setting the pointers to zero makes the system believe there is no data to
    *  read and writing will resume at address 0 overwriting any data that may
    *  have remained in the RAM.
    *
    * Side Effects:
    *  Any received data not read from the RAM or FIFO buffer will be lost.
    *
    *******************************************************************************/
    void UARTOUT_ClearRxBuffer(void) 
    {
        uint8 enableInterrupts;

        /* Clear the HW FIFO */
        enableInterrupts = CyEnterCriticalSection();
        UARTOUT_RXDATA_AUX_CTL_REG |= (uint8)  UARTOUT_RX_FIFO_CLR;
        UARTOUT_RXDATA_AUX_CTL_REG &= (uint8) ~UARTOUT_RX_FIFO_CLR;
        CyExitCriticalSection(enableInterrupts);

    #if (UARTOUT_RX_INTERRUPT_ENABLED)

        /* Protect variables that could change on interrupt. */
        UARTOUT_DisableRxInt();

        UARTOUT_rxBufferRead = 0u;
        UARTOUT_rxBufferWrite = 0u;
        UARTOUT_rxBufferLoopDetect = 0u;
        UARTOUT_rxBufferOverflow = 0u;

        UARTOUT_EnableRxInt();

    #endif /* (UARTOUT_RX_INTERRUPT_ENABLED) */

    }


    /*******************************************************************************
    * Function Name: UARTOUT_SetRxAddressMode
    ********************************************************************************
    *
    * Summary:
    *  Sets the software controlled Addressing mode used by the RX portion of the
    *  UART.
    *
    * Parameters:
    *  addressMode: Enumerated value indicating the mode of RX addressing
    *  UARTOUT__B_UART__AM_SW_BYTE_BYTE -  Software Byte-by-Byte address
    *                                               detection
    *  UARTOUT__B_UART__AM_SW_DETECT_TO_BUFFER - Software Detect to Buffer
    *                                               address detection
    *  UARTOUT__B_UART__AM_HW_BYTE_BY_BYTE - Hardware Byte-by-Byte address
    *                                               detection
    *  UARTOUT__B_UART__AM_HW_DETECT_TO_BUFFER - Hardware Detect to Buffer
    *                                               address detection
    *  UARTOUT__B_UART__AM_NONE - No address detection
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  UARTOUT_rxAddressMode - the parameter stored in this variable for
    *   the farther usage in RX ISR.
    *  UARTOUT_rxAddressDetected - set to initial state (0).
    *
    *******************************************************************************/
    void UARTOUT_SetRxAddressMode(uint8 addressMode)
                                                        
    {
        #if(UARTOUT_RXHW_ADDRESS_ENABLED)
            #if(UARTOUT_CONTROL_REG_REMOVED)
                if(0u != addressMode)
                {
                    /* Suppress compiler warning */
                }
            #else /* UARTOUT_CONTROL_REG_REMOVED */
                uint8 tmpCtrl;
                tmpCtrl = UARTOUT_CONTROL_REG & (uint8)~UARTOUT_CTRL_RXADDR_MODE_MASK;
                tmpCtrl |= (uint8)(addressMode << UARTOUT_CTRL_RXADDR_MODE0_SHIFT);
                UARTOUT_CONTROL_REG = tmpCtrl;

                #if(UARTOUT_RX_INTERRUPT_ENABLED && \
                   (UARTOUT_RXBUFFERSIZE > UARTOUT_FIFO_LENGTH) )
                    UARTOUT_rxAddressMode = addressMode;
                    UARTOUT_rxAddressDetected = 0u;
                #endif /* End UARTOUT_RXBUFFERSIZE > UARTOUT_FIFO_LENGTH*/
            #endif /* End UARTOUT_CONTROL_REG_REMOVED */
        #else /* UARTOUT_RXHW_ADDRESS_ENABLED */
            if(0u != addressMode)
            {
                /* Suppress compiler warning */
            }
        #endif /* End UARTOUT_RXHW_ADDRESS_ENABLED */
    }


    /*******************************************************************************
    * Function Name: UARTOUT_SetRxAddress1
    ********************************************************************************
    *
    * Summary:
    *  Sets the first of two hardware-detectable receiver addresses.
    *
    * Parameters:
    *  address: Address #1 for hardware address detection.
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void UARTOUT_SetRxAddress1(uint8 address) 
    {
        UARTOUT_RXADDRESS1_REG = address;
    }


    /*******************************************************************************
    * Function Name: UARTOUT_SetRxAddress2
    ********************************************************************************
    *
    * Summary:
    *  Sets the second of two hardware-detectable receiver addresses.
    *
    * Parameters:
    *  address: Address #2 for hardware address detection.
    *
    * Return:
    *  None.
    *
    *******************************************************************************/
    void UARTOUT_SetRxAddress2(uint8 address) 
    {
        UARTOUT_RXADDRESS2_REG = address;
    }

#endif  /* UARTOUT_RX_ENABLED || UARTOUT_HD_ENABLED*/


#if( (UARTOUT_TX_ENABLED) || (UARTOUT_HD_ENABLED) )
    /*******************************************************************************
    * Function Name: UARTOUT_SetTxInterruptMode
    ********************************************************************************
    *
    * Summary:
    *  Configures the TX interrupt sources to be enabled, but does not enable the
    *  interrupt.
    *
    * Parameters:
    *  intSrc: Bit field containing the TX interrupt sources to enable
    *   UARTOUT_TX_STS_COMPLETE        Interrupt on TX byte complete
    *   UARTOUT_TX_STS_FIFO_EMPTY      Interrupt when TX FIFO is empty
    *   UARTOUT_TX_STS_FIFO_FULL       Interrupt when TX FIFO is full
    *   UARTOUT_TX_STS_FIFO_NOT_FULL   Interrupt when TX FIFO is not full
    *
    * Return:
    *  None.
    *
    * Theory:
    *  Enables the output of specific status bits to the interrupt controller
    *
    *******************************************************************************/
    void UARTOUT_SetTxInterruptMode(uint8 intSrc) 
    {
        UARTOUT_TXSTATUS_MASK_REG = intSrc;
    }


    /*******************************************************************************
    * Function Name: UARTOUT_WriteTxData
    ********************************************************************************
    *
    * Summary:
    *  Places a byte of data into the transmit buffer to be sent when the bus is
    *  available without checking the TX status register. You must check status
    *  separately.
    *
    * Parameters:
    *  txDataByte: data byte
    *
    * Return:
    * None.
    *
    * Global Variables:
    *  UARTOUT_txBuffer - RAM buffer pointer for save data for transmission
    *  UARTOUT_txBufferWrite - cyclic index for write to txBuffer,
    *    incremented after each byte saved to buffer.
    *  UARTOUT_txBufferRead - cyclic index for read from txBuffer,
    *    checked to identify the condition to write to FIFO directly or to TX buffer
    *  UARTOUT_initVar - checked to identify that the component has been
    *    initialized.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    void UARTOUT_WriteTxData(uint8 txDataByte) 
    {
        /* If not Initialized then skip this function*/
        if(UARTOUT_initVar != 0u)
        {
        #if (UARTOUT_TX_INTERRUPT_ENABLED)

            /* Protect variables that could change on interrupt. */
            UARTOUT_DisableTxInt();

            if( (UARTOUT_txBufferRead == UARTOUT_txBufferWrite) &&
                ((UARTOUT_TXSTATUS_REG & UARTOUT_TX_STS_FIFO_FULL) == 0u) )
            {
                /* Add directly to the FIFO. */
                UARTOUT_TXDATA_REG = txDataByte;
            }
            else
            {
                if(UARTOUT_txBufferWrite >= UARTOUT_TX_BUFFER_SIZE)
                {
                    UARTOUT_txBufferWrite = 0u;
                }

                UARTOUT_txBuffer[UARTOUT_txBufferWrite] = txDataByte;

                /* Add to the software buffer. */
                UARTOUT_txBufferWrite++;
            }

            UARTOUT_EnableTxInt();

        #else

            /* Add directly to the FIFO. */
            UARTOUT_TXDATA_REG = txDataByte;

        #endif /*(UARTOUT_TX_INTERRUPT_ENABLED) */
        }
    }


    /*******************************************************************************
    * Function Name: UARTOUT_ReadTxStatus
    ********************************************************************************
    *
    * Summary:
    *  Reads the status register for the TX portion of the UART.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Contents of the status register
    *
    * Theory:
    *  This function reads the TX status register, which is cleared on read.
    *  It is up to the user to handle all bits in this return value accordingly,
    *  even if the bit was not enabled as an interrupt source the event happened
    *  and must be handled accordingly.
    *
    *******************************************************************************/
    uint8 UARTOUT_ReadTxStatus(void) 
    {
        return(UARTOUT_TXSTATUS_REG);
    }


    /*******************************************************************************
    * Function Name: UARTOUT_PutChar
    ********************************************************************************
    *
    * Summary:
    *  Puts a byte of data into the transmit buffer to be sent when the bus is
    *  available. This is a blocking API that waits until the TX buffer has room to
    *  hold the data.
    *
    * Parameters:
    *  txDataByte: Byte containing the data to transmit
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  UARTOUT_txBuffer - RAM buffer pointer for save data for transmission
    *  UARTOUT_txBufferWrite - cyclic index for write to txBuffer,
    *     checked to identify free space in txBuffer and incremented after each byte
    *     saved to buffer.
    *  UARTOUT_txBufferRead - cyclic index for read from txBuffer,
    *     checked to identify free space in txBuffer.
    *  UARTOUT_initVar - checked to identify that the component has been
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Allows the user to transmit any byte of data in a single transfer
    *
    *******************************************************************************/
    void UARTOUT_PutChar(uint8 txDataByte) 
    {
    #if (UARTOUT_TX_INTERRUPT_ENABLED)
        /* The temporary output pointer is used since it takes two instructions
        *  to increment with a wrap, and we can't risk doing that with the real
        *  pointer and getting an interrupt in between instructions.
        */
        uint8 locTxBufferWrite;
        uint8 locTxBufferRead;

        do
        { /* Block if software buffer is full, so we don't overwrite. */

        #if ((UARTOUT_TX_BUFFER_SIZE > UARTOUT_MAX_BYTE_VALUE) && (CY_PSOC3))
            /* Disable TX interrupt to protect variables from modification */
            UARTOUT_DisableTxInt();
        #endif /* (UARTOUT_TX_BUFFER_SIZE > UARTOUT_MAX_BYTE_VALUE) && (CY_PSOC3) */

            locTxBufferWrite = UARTOUT_txBufferWrite;
            locTxBufferRead  = UARTOUT_txBufferRead;

        #if ((UARTOUT_TX_BUFFER_SIZE > UARTOUT_MAX_BYTE_VALUE) && (CY_PSOC3))
            /* Enable interrupt to continue transmission */
            UARTOUT_EnableTxInt();
        #endif /* (UARTOUT_TX_BUFFER_SIZE > UARTOUT_MAX_BYTE_VALUE) && (CY_PSOC3) */
        }
        while( (locTxBufferWrite < locTxBufferRead) ? (locTxBufferWrite == (locTxBufferRead - 1u)) :
                                ((locTxBufferWrite - locTxBufferRead) ==
                                (uint8)(UARTOUT_TX_BUFFER_SIZE - 1u)) );

        if( (locTxBufferRead == locTxBufferWrite) &&
            ((UARTOUT_TXSTATUS_REG & UARTOUT_TX_STS_FIFO_FULL) == 0u) )
        {
            /* Add directly to the FIFO */
            UARTOUT_TXDATA_REG = txDataByte;
        }
        else
        {
            if(locTxBufferWrite >= UARTOUT_TX_BUFFER_SIZE)
            {
                locTxBufferWrite = 0u;
            }
            /* Add to the software buffer. */
            UARTOUT_txBuffer[locTxBufferWrite] = txDataByte;
            locTxBufferWrite++;

            /* Finally, update the real output pointer */
        #if ((UARTOUT_TX_BUFFER_SIZE > UARTOUT_MAX_BYTE_VALUE) && (CY_PSOC3))
            UARTOUT_DisableTxInt();
        #endif /* (UARTOUT_TX_BUFFER_SIZE > UARTOUT_MAX_BYTE_VALUE) && (CY_PSOC3) */

            UARTOUT_txBufferWrite = locTxBufferWrite;

        #if ((UARTOUT_TX_BUFFER_SIZE > UARTOUT_MAX_BYTE_VALUE) && (CY_PSOC3))
            UARTOUT_EnableTxInt();
        #endif /* (UARTOUT_TX_BUFFER_SIZE > UARTOUT_MAX_BYTE_VALUE) && (CY_PSOC3) */

            if(0u != (UARTOUT_TXSTATUS_REG & UARTOUT_TX_STS_FIFO_EMPTY))
            {
                /* Trigger TX interrupt to send software buffer */
                UARTOUT_SetPendingTxInt();
            }
        }

    #else

        while((UARTOUT_TXSTATUS_REG & UARTOUT_TX_STS_FIFO_FULL) != 0u)
        {
            /* Wait for room in the FIFO */
        }

        /* Add directly to the FIFO */
        UARTOUT_TXDATA_REG = txDataByte;

    #endif /* UARTOUT_TX_INTERRUPT_ENABLED */
    }


    /*******************************************************************************
    * Function Name: UARTOUT_PutString
    ********************************************************************************
    *
    * Summary:
    *  Sends a NULL terminated string to the TX buffer for transmission.
    *
    * Parameters:
    *  string[]: Pointer to the null terminated string array residing in RAM or ROM
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  UARTOUT_initVar - checked to identify that the component has been
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  If there is not enough memory in the TX buffer for the entire string, this
    *  function blocks until the last character of the string is loaded into the
    *  TX buffer.
    *
    *******************************************************************************/
    void UARTOUT_PutString(const char8 string[]) 
    {
        uint16 bufIndex = 0u;

        /* If not Initialized then skip this function */
        if(UARTOUT_initVar != 0u)
        {
            /* This is a blocking function, it will not exit until all data is sent */
            while(string[bufIndex] != (char8) 0)
            {
                UARTOUT_PutChar((uint8)string[bufIndex]);
                bufIndex++;
            }
        }
    }


    /*******************************************************************************
    * Function Name: UARTOUT_PutArray
    ********************************************************************************
    *
    * Summary:
    *  Places N bytes of data from a memory array into the TX buffer for
    *  transmission.
    *
    * Parameters:
    *  string[]: Address of the memory array residing in RAM or ROM.
    *  byteCount: Number of bytes to be transmitted. The type depends on TX Buffer
    *             Size parameter.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  UARTOUT_initVar - checked to identify that the component has been
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  If there is not enough memory in the TX buffer for the entire string, this
    *  function blocks until the last character of the string is loaded into the
    *  TX buffer.
    *
    *******************************************************************************/
    void UARTOUT_PutArray(const uint8 string[], uint8 byteCount)
                                                                    
    {
        uint8 bufIndex = 0u;

        /* If not Initialized then skip this function */
        if(UARTOUT_initVar != 0u)
        {
            while(bufIndex < byteCount)
            {
                UARTOUT_PutChar(string[bufIndex]);
                bufIndex++;
            }
        }
    }


    /*******************************************************************************
    * Function Name: UARTOUT_PutCRLF
    ********************************************************************************
    *
    * Summary:
    *  Writes a byte of data followed by a carriage return (0x0D) and line feed
    *  (0x0A) to the transmit buffer.
    *
    * Parameters:
    *  txDataByte: Data byte to transmit before the carriage return and line feed.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  UARTOUT_initVar - checked to identify that the component has been
    *     initialized.
    *
    * Reentrant:
    *  No.
    *
    *******************************************************************************/
    void UARTOUT_PutCRLF(uint8 txDataByte) 
    {
        /* If not Initialized then skip this function */
        if(UARTOUT_initVar != 0u)
        {
            UARTOUT_PutChar(txDataByte);
            UARTOUT_PutChar(0x0Du);
            UARTOUT_PutChar(0x0Au);
        }
    }


    /*******************************************************************************
    * Function Name: UARTOUT_GetTxBufferSize
    ********************************************************************************
    *
    * Summary:
    *  Returns the number of bytes in the TX buffer which are waiting to be 
    *  transmitted.
    *  * TX software buffer is disabled (TX Buffer Size parameter is equal to 4): 
    *    returns 0 for empty TX FIFO, 1 for not full TX FIFO or 4 for full TX FIFO.
    *  * TX software buffer is enabled: returns the number of bytes in the TX 
    *    software buffer which are waiting to be transmitted. Bytes available in the
    *    TX FIFO do not count.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  Number of bytes used in the TX buffer. Return value type depends on the TX 
    *  Buffer Size parameter.
    *
    * Global Variables:
    *  UARTOUT_txBufferWrite - used to calculate left space.
    *  UARTOUT_txBufferRead - used to calculate left space.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Allows the user to find out how full the TX Buffer is.
    *
    *******************************************************************************/
    uint8 UARTOUT_GetTxBufferSize(void)
                                                            
    {
        uint8 size;

    #if (UARTOUT_TX_INTERRUPT_ENABLED)

        /* Protect variables that could change on interrupt. */
        UARTOUT_DisableTxInt();

        if(UARTOUT_txBufferRead == UARTOUT_txBufferWrite)
        {
            size = 0u;
        }
        else if(UARTOUT_txBufferRead < UARTOUT_txBufferWrite)
        {
            size = (UARTOUT_txBufferWrite - UARTOUT_txBufferRead);
        }
        else
        {
            size = (UARTOUT_TX_BUFFER_SIZE - UARTOUT_txBufferRead) +
                    UARTOUT_txBufferWrite;
        }

        UARTOUT_EnableTxInt();

    #else

        size = UARTOUT_TXSTATUS_REG;

        /* Is the fifo is full. */
        if((size & UARTOUT_TX_STS_FIFO_FULL) != 0u)
        {
            size = UARTOUT_FIFO_LENGTH;
        }
        else if((size & UARTOUT_TX_STS_FIFO_EMPTY) != 0u)
        {
            size = 0u;
        }
        else
        {
            /* We only know there is data in the fifo. */
            size = 1u;
        }

    #endif /* (UARTOUT_TX_INTERRUPT_ENABLED) */

    return(size);
    }


    /*******************************************************************************
    * Function Name: UARTOUT_ClearTxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Clears all data from the TX buffer and hardware TX FIFO.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  UARTOUT_txBufferWrite - cleared to zero.
    *  UARTOUT_txBufferRead - cleared to zero.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  Setting the pointers to zero makes the system believe there is no data to
    *  read and writing will resume at address 0 overwriting any data that may have
    *  remained in the RAM.
    *
    * Side Effects:
    *  Data waiting in the transmit buffer is not sent; a byte that is currently
    *  transmitting finishes transmitting.
    *
    *******************************************************************************/
    void UARTOUT_ClearTxBuffer(void) 
    {
        uint8 enableInterrupts;

        enableInterrupts = CyEnterCriticalSection();
        /* Clear the HW FIFO */
        UARTOUT_TXDATA_AUX_CTL_REG |= (uint8)  UARTOUT_TX_FIFO_CLR;
        UARTOUT_TXDATA_AUX_CTL_REG &= (uint8) ~UARTOUT_TX_FIFO_CLR;
        CyExitCriticalSection(enableInterrupts);

    #if (UARTOUT_TX_INTERRUPT_ENABLED)

        /* Protect variables that could change on interrupt. */
        UARTOUT_DisableTxInt();

        UARTOUT_txBufferRead = 0u;
        UARTOUT_txBufferWrite = 0u;

        /* Enable Tx interrupt. */
        UARTOUT_EnableTxInt();

    #endif /* (UARTOUT_TX_INTERRUPT_ENABLED) */
    }


    /*******************************************************************************
    * Function Name: UARTOUT_SendBreak
    ********************************************************************************
    *
    * Summary:
    *  Transmits a break signal on the bus.
    *
    * Parameters:
    *  uint8 retMode:  Send Break return mode. See the following table for options.
    *   UARTOUT_SEND_BREAK - Initialize registers for break, send the Break
    *       signal and return immediately.
    *   UARTOUT_WAIT_FOR_COMPLETE_REINIT - Wait until break transmission is
    *       complete, reinitialize registers to normal transmission mode then return
    *   UARTOUT_REINIT - Reinitialize registers to normal transmission mode
    *       then return.
    *   UARTOUT_SEND_WAIT_REINIT - Performs both options: 
    *      UARTOUT_SEND_BREAK and UARTOUT_WAIT_FOR_COMPLETE_REINIT.
    *      This option is recommended for most cases.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  UARTOUT_initVar - checked to identify that the component has been
    *     initialized.
    *  txPeriod - static variable, used for keeping TX period configuration.
    *
    * Reentrant:
    *  No.
    *
    * Theory:
    *  SendBreak function initializes registers to send 13-bit break signal. It is
    *  important to return the registers configuration to normal for continue 8-bit
    *  operation.
    *  There are 3 variants for this API usage:
    *  1) SendBreak(3) - function will send the Break signal and take care on the
    *     configuration returning. Function will block CPU until transmission
    *     complete.
    *  2) User may want to use blocking time if UART configured to the low speed
    *     operation
    *     Example for this case:
    *     SendBreak(0);     - initialize Break signal transmission
    *         Add your code here to use CPU time
    *     SendBreak(1);     - complete Break operation
    *  3) Same to 2) but user may want to initialize and use the interrupt to
    *     complete break operation.
    *     Example for this case:
    *     Initialize TX interrupt with "TX - On TX Complete" parameter
    *     SendBreak(0);     - initialize Break signal transmission
    *         Add your code here to use CPU time
    *     When interrupt appear with UARTOUT_TX_STS_COMPLETE status:
    *     SendBreak(2);     - complete Break operation
    *
    * Side Effects:
    *  The UARTOUT_SendBreak() function initializes registers to send a
    *  break signal.
    *  Break signal length depends on the break signal bits configuration.
    *  The register configuration should be reinitialized before normal 8-bit
    *  communication can continue.
    *
    *******************************************************************************/
    void UARTOUT_SendBreak(uint8 retMode) 
    {

        /* If not Initialized then skip this function*/
        if(UARTOUT_initVar != 0u)
        {
            /* Set the Counter to 13-bits and transmit a 00 byte */
            /* When that is done then reset the counter value back */
            uint8 tmpStat;

        #if(UARTOUT_HD_ENABLED) /* Half Duplex mode*/

            if( (retMode == UARTOUT_SEND_BREAK) ||
                (retMode == UARTOUT_SEND_WAIT_REINIT ) )
            {
                /* CTRL_HD_SEND_BREAK - sends break bits in HD mode */
                UARTOUT_WriteControlRegister(UARTOUT_ReadControlRegister() |
                                                      UARTOUT_CTRL_HD_SEND_BREAK);
                /* Send zeros */
                UARTOUT_TXDATA_REG = 0u;

                do /* Wait until transmit starts */
                {
                    tmpStat = UARTOUT_TXSTATUS_REG;
                }
                while((tmpStat & UARTOUT_TX_STS_FIFO_EMPTY) != 0u);
            }

            if( (retMode == UARTOUT_WAIT_FOR_COMPLETE_REINIT) ||
                (retMode == UARTOUT_SEND_WAIT_REINIT) )
            {
                do /* Wait until transmit complete */
                {
                    tmpStat = UARTOUT_TXSTATUS_REG;
                }
                while(((uint8)~tmpStat & UARTOUT_TX_STS_COMPLETE) != 0u);
            }

            if( (retMode == UARTOUT_WAIT_FOR_COMPLETE_REINIT) ||
                (retMode == UARTOUT_REINIT) ||
                (retMode == UARTOUT_SEND_WAIT_REINIT) )
            {
                UARTOUT_WriteControlRegister(UARTOUT_ReadControlRegister() &
                                              (uint8)~UARTOUT_CTRL_HD_SEND_BREAK);
            }

        #else /* UARTOUT_HD_ENABLED Full Duplex mode */

            static uint8 txPeriod;

            if( (retMode == UARTOUT_SEND_BREAK) ||
                (retMode == UARTOUT_SEND_WAIT_REINIT) )
            {
                /* CTRL_HD_SEND_BREAK - skip to send parity bit at Break signal in Full Duplex mode */
                #if( (UARTOUT_PARITY_TYPE != UARTOUT__B_UART__NONE_REVB) || \
                                    (UARTOUT_PARITY_TYPE_SW != 0u) )
                    UARTOUT_WriteControlRegister(UARTOUT_ReadControlRegister() |
                                                          UARTOUT_CTRL_HD_SEND_BREAK);
                #endif /* End UARTOUT_PARITY_TYPE != UARTOUT__B_UART__NONE_REVB  */

                #if(UARTOUT_TXCLKGEN_DP)
                    txPeriod = UARTOUT_TXBITCLKTX_COMPLETE_REG;
                    UARTOUT_TXBITCLKTX_COMPLETE_REG = UARTOUT_TXBITCTR_BREAKBITS;
                #else
                    txPeriod = UARTOUT_TXBITCTR_PERIOD_REG;
                    UARTOUT_TXBITCTR_PERIOD_REG = UARTOUT_TXBITCTR_BREAKBITS8X;
                #endif /* End UARTOUT_TXCLKGEN_DP */

                /* Send zeros */
                UARTOUT_TXDATA_REG = 0u;

                do /* Wait until transmit starts */
                {
                    tmpStat = UARTOUT_TXSTATUS_REG;
                }
                while((tmpStat & UARTOUT_TX_STS_FIFO_EMPTY) != 0u);
            }

            if( (retMode == UARTOUT_WAIT_FOR_COMPLETE_REINIT) ||
                (retMode == UARTOUT_SEND_WAIT_REINIT) )
            {
                do /* Wait until transmit complete */
                {
                    tmpStat = UARTOUT_TXSTATUS_REG;
                }
                while(((uint8)~tmpStat & UARTOUT_TX_STS_COMPLETE) != 0u);
            }

            if( (retMode == UARTOUT_WAIT_FOR_COMPLETE_REINIT) ||
                (retMode == UARTOUT_REINIT) ||
                (retMode == UARTOUT_SEND_WAIT_REINIT) )
            {

            #if(UARTOUT_TXCLKGEN_DP)
                UARTOUT_TXBITCLKTX_COMPLETE_REG = txPeriod;
            #else
                UARTOUT_TXBITCTR_PERIOD_REG = txPeriod;
            #endif /* End UARTOUT_TXCLKGEN_DP */

            #if( (UARTOUT_PARITY_TYPE != UARTOUT__B_UART__NONE_REVB) || \
                 (UARTOUT_PARITY_TYPE_SW != 0u) )
                UARTOUT_WriteControlRegister(UARTOUT_ReadControlRegister() &
                                                      (uint8) ~UARTOUT_CTRL_HD_SEND_BREAK);
            #endif /* End UARTOUT_PARITY_TYPE != NONE */
            }
        #endif    /* End UARTOUT_HD_ENABLED */
        }
    }


    /*******************************************************************************
    * Function Name: UARTOUT_SetTxAddressMode
    ********************************************************************************
    *
    * Summary:
    *  Configures the transmitter to signal the next bytes is address or data.
    *
    * Parameters:
    *  addressMode: 
    *       UARTOUT_SET_SPACE - Configure the transmitter to send the next
    *                                    byte as a data.
    *       UARTOUT_SET_MARK  - Configure the transmitter to send the next
    *                                    byte as an address.
    *
    * Return:
    *  None.
    *
    * Side Effects:
    *  This function sets and clears UARTOUT_CTRL_MARK bit in the Control
    *  register.
    *
    *******************************************************************************/
    void UARTOUT_SetTxAddressMode(uint8 addressMode) 
    {
        /* Mark/Space sending enable */
        if(addressMode != 0u)
        {
        #if( UARTOUT_CONTROL_REG_REMOVED == 0u )
            UARTOUT_WriteControlRegister(UARTOUT_ReadControlRegister() |
                                                  UARTOUT_CTRL_MARK);
        #endif /* End UARTOUT_CONTROL_REG_REMOVED == 0u */
        }
        else
        {
        #if( UARTOUT_CONTROL_REG_REMOVED == 0u )
            UARTOUT_WriteControlRegister(UARTOUT_ReadControlRegister() &
                                                  (uint8) ~UARTOUT_CTRL_MARK);
        #endif /* End UARTOUT_CONTROL_REG_REMOVED == 0u */
        }
    }

#endif  /* EndUARTOUT_TX_ENABLED */

#if(UARTOUT_HD_ENABLED)


    /*******************************************************************************
    * Function Name: UARTOUT_LoadRxConfig
    ********************************************************************************
    *
    * Summary:
    *  Loads the receiver configuration in half duplex mode. After calling this
    *  function, the UART is ready to receive data.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Side Effects:
    *  Valid only in half duplex mode. You must make sure that the previous
    *  transaction is complete and it is safe to unload the transmitter
    *  configuration.
    *
    *******************************************************************************/
    void UARTOUT_LoadRxConfig(void) 
    {
        UARTOUT_WriteControlRegister(UARTOUT_ReadControlRegister() &
                                                (uint8)~UARTOUT_CTRL_HD_SEND);
        UARTOUT_RXBITCTR_PERIOD_REG = UARTOUT_HD_RXBITCTR_INIT;

    #if (UARTOUT_RX_INTERRUPT_ENABLED)
        /* Enable RX interrupt after set RX configuration */
        UARTOUT_SetRxInterruptMode(UARTOUT_INIT_RX_INTERRUPTS_MASK);
    #endif /* (UARTOUT_RX_INTERRUPT_ENABLED) */
    }


    /*******************************************************************************
    * Function Name: UARTOUT_LoadTxConfig
    ********************************************************************************
    *
    * Summary:
    *  Loads the transmitter configuration in half duplex mode. After calling this
    *  function, the UART is ready to transmit data.
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Side Effects:
    *  Valid only in half duplex mode. You must make sure that the previous
    *  transaction is complete and it is safe to unload the receiver configuration.
    *
    *******************************************************************************/
    void UARTOUT_LoadTxConfig(void) 
    {
    #if (UARTOUT_RX_INTERRUPT_ENABLED)
        /* Disable RX interrupts before set TX configuration */
        UARTOUT_SetRxInterruptMode(0u);
    #endif /* (UARTOUT_RX_INTERRUPT_ENABLED) */

        UARTOUT_WriteControlRegister(UARTOUT_ReadControlRegister() | UARTOUT_CTRL_HD_SEND);
        UARTOUT_RXBITCTR_PERIOD_REG = UARTOUT_HD_TXBITCTR_INIT;
    }

#endif  /* UARTOUT_HD_ENABLED */


/* [] END OF FILE */
